//+------------------------------------------------------------------+
//|                                                       HULLX3.mq4 |
//|                                                              TTK |
//|                                     https://youtu.be/LjIAzKo62MQ |
//+------------------------------------------------------------------+
#property copyright "TTK"
#property link      "https://youtu.be/LjIAzKo62MQ"
#property version   "1.00"
#property strict
#property indicator_chart_window
#property indicator_buffers 3

#property indicator_label1  "HULL_X3"
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrMediumSeaGreen
#property indicator_width1  2

#property indicator_label2  "HULL - SLOPE DOWN"
#property indicator_type2   DRAW_LINE
#property indicator_color2  clrRed
#property indicator_width2  2

#property indicator_label3  "HULL - SLOPE DOWN"
#property indicator_type3   DRAW_LINE
#property indicator_color3  clrRed
#property indicator_width3  2

extern string HULL_X3_SETTINGS;
input int inp_period  = 20;                             //ENTER PERIOD
input double inp_divisor = 2.0;                         //ENTER DIVISOR (I.E. SPEED)
input ENUM_APPLIED_PRICE inp_price = PRICE_CLOSE;       //ENTER APPLIED PRICE

double VAL[];
double VALD_A[];
double VALD_B[];
double VALD_C[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CHull
  {
private :
   int               m_full_period;
   int               m_half_period;
   int               m_sqrt_period;
   int               m_array_size;
   double            m_weight_1;
   double            m_weight_2;
   double            m_weight_3;

   struct sHullArrayStruct
     {
      double         value;
      double         value_3;
      double         w_sum_1;
      double         w_sum_2;
      double         w_sum_3;
      double         l_sum_1;
      double         l_sum_2;
      double         l_sum_3;
     };

   sHullArrayStruct  m_array[];

public :
                     CHull() : m_full_period(1), m_half_period(1), m_sqrt_period(1), m_array_size(-1) {                     }
                    ~CHull()                                                                          { ArrayFree(m_array); }

   bool              init(int period, double divisor)
     {
      m_full_period = (int)(period > 1 ? period : 1);

      m_half_period = (int)(m_full_period > 1 ? m_full_period / (divisor > 1 ? divisor : 1) : 1);

      m_sqrt_period = (int) MathSqrt(m_full_period);

      m_array_size  = -1;

      m_weight_1 = m_weight_2 = m_weight_3 = 1;

      return(true);
     }

   double            calculate(double value, int i, int bars)
     {
      if(m_array_size < bars)
        {
         m_array_size = ArrayResize(m_array, (bars + 500));

         if(m_array_size < bars)
           {
            return(0);
           }
        }

      m_array[i].value = value;

      if(i > m_full_period)
        {
         m_array[i].w_sum_1 = m_array[i - 1].w_sum_1 + value * m_half_period - m_array[i - 1].l_sum_1;

         m_array[i].l_sum_1 = m_array[i - 1].l_sum_1 + value - m_array[i - m_half_period].value;

         m_array[i].w_sum_2 = m_array[i - 1].w_sum_2 + value * m_full_period - m_array[i - 1].l_sum_2;

         m_array[i].l_sum_2 = m_array[i - 1].l_sum_2 + value - m_array[i - m_full_period].value;
        }
      else
        {
         m_array[i].w_sum_1 =
            m_array[i].w_sum_2 =
               m_array[i].l_sum_1 =
                  m_array[i].l_sum_2 =
                     m_weight_1 =
                        m_weight_2 =
                           0;

         for(int k = 0, w_1 = m_half_period, w_2 = m_full_period; w_2 > 0 && i >= k; k++, w_1--, w_2--)
           {
            if(w_1 > 0)
              {
               m_array[i].w_sum_1 += m_array[i - k].value * w_1;

               m_array[i].l_sum_1 += m_array[i - k].value;

               m_weight_1 += w_1;
              }

            m_array[i].w_sum_2 += m_array[i - k].value * w_2;

            m_array[i].l_sum_2 += m_array[i - k].value;

            m_weight_2 += w_2;
           }
        }

      m_array[i].value_3 = 2.0 * m_array[i].w_sum_1 / m_weight_1 - m_array[i].w_sum_2 / m_weight_2;

      if(i > m_sqrt_period)
        {
         m_array[i].w_sum_3 = m_array[i - 1].w_sum_3 + m_array[i].value_3 * m_sqrt_period - m_array[i - 1].l_sum_3;

         m_array[i].l_sum_3 = m_array[i - 1].l_sum_3 + m_array[i].value_3 - m_array[i - m_sqrt_period].value_3;
        }
      else
        {
         m_array[i].w_sum_3 =
            m_array[i].l_sum_3 =
               m_weight_3 =
                  0;

         for(int k = 0, w_3 = m_sqrt_period; w_3 > 0 && i >= k; k++, w_3--)
           {
            m_array[i].w_sum_3 += m_array[i - k].value_3 * w_3;
            m_array[i].l_sum_3 += m_array[i - k].value_3;
            m_weight_3 += w_3;
           }
        }

      return(m_array[i].w_sum_3 / m_weight_3);
     }
  };

CHull iHull;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void i_clean_point(int i, double &first[], double &second[])
  {
//---
   if(i >= (Bars - 3))
     {
      return;
     }

   if((second[i] != EMPTY_VALUE) && (second[i + 1] != EMPTY_VALUE))
     {
      second[i + 1] = EMPTY_VALUE;
     }
   else
      if((first[i] != EMPTY_VALUE) && (first[i + 1] != EMPTY_VALUE) && (first[i + 2] == EMPTY_VALUE))
        {
         first[i + 1] = EMPTY_VALUE;
        }
//---
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void i_plot_point(int i, double &first[], double &second[], double &from[])
  {
//---
   if(i >= (Bars - 2))
     {
      return;
     }

   if(first[i + 1] == EMPTY_VALUE)
     {
      if(first[i + 2] == EMPTY_VALUE)
        {
         first[i] = from[i];

         first[i + 1]  = from[i + 1];

         second[i] = EMPTY_VALUE;
        }
      else
        {
         second[i] =  from[i];

         second[i + 1] = from[i + 1];

         first[i] = EMPTY_VALUE;
        }
     }
   else
     {
      first[i]  = from[i];

      second[i] = EMPTY_VALUE;
     }
//---
  }

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   IndicatorBuffers(4);
//---
   SetIndexBuffer(0, VAL, INDICATOR_DATA);
   SetIndexBuffer(1, VALD_A, INDICATOR_DATA);
   SetIndexBuffer(2, VALD_B, INDICATOR_DATA);
   SetIndexBuffer(3, VALD_C);
//---
   iHull.init(inp_period, inp_divisor);
//---
   IndicatorSetString(INDICATOR_SHORTNAME, "HULL_X3 (" + (string)inp_period + "," + (string)inp_divisor + ")");
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   int i = rates_total - prev_calculated + 1;

   if(i >= rates_total)
     {
      i = rates_total - 1;
     }

   if(VALD_C[i] == -1)
     {
      i_clean_point(i, VALD_A, VALD_B);
     }

   for(; i >= 0 && !_StopFlag; i--)
     {
      VAL[i]   = iHull.calculate(iMA(NULL, 0, 1, 0, MODE_EMA, inp_price, i), (rates_total - i - 1), rates_total);

      VALD_C[i]  = (i < (rates_total - 1)) ? (VAL[i] > VAL[i + 1]) ? 1 : (VAL[i] < VAL[i + 1]) ? - 1 : VALD_C[i + 1] : 0;

      VALD_A[i] = VALD_B[i] = EMPTY_VALUE;

      /*if(VALD_C[i] == 1)
        {
         ObjectCreate(
            "BUY_ZONE_" + (string)i,
            OBJ_ARROW,
            0,
            Time[i],
            Low[i]
         );

         ObjectSetInteger(
            ChartID(),
            "BUY_ZONE_" + (string)i,
            OBJPROP_ARROWCODE,
            233
         );

         ObjectSetInteger(
            ChartID(),
            "BUY_ZONE_" + (string)i,
            OBJPROP_COLOR,
            clrMediumSeaGreen
         );

         ObjectSetInteger(
            ChartID(),
            "BUY_ZONE_" + (string)i,
            OBJPROP_SELECTABLE,
            FALSE
         );
        }
      else if(VALD_C[i] == -1)
        {
         ObjectCreate(
            "SELL_ZONE_" + (string)i,
            OBJ_ARROW,
            0,
            Time[i],
            High[i]
         );

         ObjectSetInteger(
            ChartID(),
            "SELL_ZONE_" + (string)i,
            OBJPROP_ARROWCODE,
            234
         );

         ObjectSetInteger(
            ChartID(),
            "SELL_ZONE_" + (string)i,
            OBJPROP_COLOR,
            clrRed
         );

         ObjectSetInteger(
            ChartID(),
            "SELL_ZONE_" + (string)i,
            OBJPROP_SELECTABLE,
            FALSE
         );
        }*/

      if(VALD_C[i] == -1)
        {
         i_plot_point(i, VALD_A, VALD_B, VAL);
        }
     }
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Timer function                                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
//---

  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---

  }
//+------------------------------------------------------------------+
